unit UPageAdmin;

{$mode delphi}

interface

uses
  Classes, SysUtils, FileUtil, UWebPage, UHTTPServer;

type

  { TPageAdmin }

  TPageAdmin = class(TWebPage)
    procedure DataModuleProduce(HandlerData: THTTPHandlerData);
  private
    procedure HandleModuleList(HandlerData: THTTPHandlerData);
    procedure HandleInstall(HandlerData: THTTPHandlerData);
    procedure HandleUninstall(HandlerData: THTTPHandlerData);
  public
    { public declarations }
  end;

var
  PageAdmin: TPageAdmin;

const
  BooleanText: array[Boolean] of string = ('No', 'Yes');


implementation

uses
  UCore, UXmlClasses, UHtmlClasses, UUtils, USqlDatabase, UModularSystem;

{$R *.lfm}

{ TPageAdmin }

procedure TPageAdmin.DataModuleProduce(HandlerData: THTTPHandlerData);
var
  TextBlock: THtmlString;
  I: Integer;
  PageName: string;
begin
  with TWebSession(HandlerData) do begin
    if Request.Path.Count > 1 then PageName := Request.Path[1]
      else PageName := '';
    with HtmlDocument.Body do begin
      if PageName = '' then PageName := 'modulelist';
      if PageName = 'modulelist' then HandleModuleList(HandlerData)
      else if PageName = 'install' then HandleInstall(HandlerData)
      else if PageName = 'uninstall' then HandleUninstall(HandlerData);
    end;
    GeneratePage(Self);
  end;
end;

procedure TPageAdmin.HandleModuleList(HandlerData: THTTPHandlerData);
var
  TextBlock: THtmlString;
  I: Integer;
begin
  with TWebSession(HandlerData) do begin
    with HtmlDocument.Body, THtmlString(SubItems.AddNew(THtmlString.Create)) do begin
      Text := Text + '<strong>Registred modules:</strong><br/>' +
        '<table class="WideTable"><tr><th>Name</th><th>Installed</th><th>Title</th><th>Version</th>' +
        '<th>License</th><th>Author</th><th>Decsription</th><th>Dependencies</th><th>Actions</th></tr>';
      for I := 0 to Core.ModuleManager.Modules.Count - 1 do
      with TModule(Core.ModuleManager.Modules[I]) do begin
        Text := Text + '<tr><td>' + Identification + '</td><td>' + BooleanText[Installed] + '</td><td>' + Title +
        '</td><td>' + Version + '</td><td>' + License + '</td><td>' + Author +
        '</td><td>' + Description.Text + '</td><td>' + Dependencies.Text + '</td><td>';
        if Installed then Text := Text + MakeLink('Uninstall', NavigationLink('/administration/uninstall?name=' + Identification))
          else Text := Text + MakeLink('Install', NavigationLink('/administration/install?name=' + Identification));
        Text := Text + '</td></tr>';
      end;
    end;
  end;
end;

procedure TPageAdmin.HandleInstall(HandlerData: THTTPHandlerData);
var
  ModuleName: string;
  Module: TModule;
begin
  with TWebSession(HandlerData) do begin
    with HtmlDocument.Body, THtmlString(SubItems.AddNew(THtmlString.Create)) do begin
      if Request.Query.IndexOfName('name') <> -1 then begin
        ModuleName := Request.Query.Values['name'];
        Module := Core.ModuleManager.FindModuleByName(ModuleName);
        if Assigned(Module) then begin
          Text := 'Installing module ' + ModuleName;
          Module.Install;
        end else Text := Text + 'Module ' + ModuleName + ' not found';
      end else Text := Text + 'Module not specified';
      Text := Text + '<br/><br/>';
      HandleModuleList(HandlerData);
    end;
  end;
end;

procedure TPageAdmin.HandleUninstall(HandlerData: THTTPHandlerData);
var
  ModuleName: string;
  Module: TModule;
begin
  with TWebSession(HandlerData) do begin
    with HtmlDocument.Body, THtmlString(SubItems.AddNew(THtmlString.Create)) do begin
      if Request.Query.IndexOfName('name') <> -1 then begin
        ModuleName := Request.Query.Values['name'];
        Module := Core.ModuleManager.FindModuleByName(ModuleName);
        if Assigned(Module) then begin
          Text := 'Uninstalling module ' + ModuleName;
          Module.Uninstall;
        end else Text := Text + 'Module ' + ModuleName + ' not found';
      end else Text := Text + 'Module not specified';
      Text := Text + '<br/><br/>';
      HandleModuleList(HandlerData);
    end;
  end;
end;

end.

